<?php

namespace Drupal\diba_integration_vus\EventSubscriber;

use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\diba_integration_vus\Webservice\VusManager;

/**
 * Subscribe to kernel request event to check authentication.
 */
class VusPostSubscriber implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The VUS manager.
   *
   * @var \Drupal\diba_integration_vus\Webservice\VusManager
   */
  protected $vusManager;

  /**
   * Constructs a VusPostSubscriber object.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\diba_integration_vus\Webservice\VusManager $vusManager
   *   The VUS manager.
   */
  public function __construct(AccountProxyInterface $currentUser, ConfigFactoryInterface $configFactory, RequestStack $requestStack, VusManager $vusManager) {
    $this->currentUser = $currentUser;
    $this->configFactory = $configFactory;
    $this->requestStack = $requestStack;
    $this->vusManager = $vusManager;
  }

  /**
   * Validates the user through VUS POST parameters.
   */
  public function checkForLoginSubmit(RequestEvent $event) {
    // Only login if user is not logged in.
    if ($this->currentUser->isAnonymous()) {
      // Automatic login if POST validation is allowed.
      $config = $this->configFactory->get('diba_integration_vus.settings');
      if ($config->get('diba_integration_vus_post') && $config->get('diba_integration_vus_validation') > 0) {
        // Ensure we receive both VUS variables.
        $request = $this->requestStack->getCurrentRequest();
        $vus_user = $request->request->get('user');
        $vus_pass = $request->request->get('pass');
        if (!empty($vus_user) && !empty($vus_pass)) {
          // Retrieve the user. Remove OPS$ from the user if it exists.
          $vus_user = Html::escape(str_replace('ops$', '', strtolower(trim($vus_user))));
          $vus_pass = Html::escape(trim($vus_pass));
          $vus_app = $config->get('diba_integration_vus_ws_app');

          $user = $this->vusManager->validateUser($vus_user, $vus_pass, $vus_app);
          if (!empty($user)) {
            $this->vusManager->login($user);
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::REQUEST][] = ['checkForLoginSubmit'];
    return $events;
  }

}
