<?php

namespace Drupal\Tests\diba_integration\Functional;

use Drupal\diba_integration_vus\Webservice\VusManager;

/**
 * Tests user login with different VUS validation settings.
 *
 * @group diba_integration
 */
class VusLoginTest extends DibaIntegrationTestBase {

  /**
   * Tests user login with VALIDACIO_DRUPAL setting.
   */
  public function testLoginWithValidacioDrupal() {
    // Set the VUS validation setting to VALIDACIO_DRUPAL.
    $this->config('diba_integration_vus.settings')
      ->set('diba_integration_vus_validation', VusManager::VALIDACIO_DRUPAL)
      ->save();

    // Create a user.
    $user = $this->drupalCreateUser();

    // Log out the admin user.
    $this->drupalLogout();

    // Attempt to log in with the created user.
    $this->drupalLogin($user);
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('user/' . $user->id());
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests user login with VALIDACIO_VUS_I_DRUPAL setting.
   */
  public function testLoginWithValidacioVusAndDrupal() {
    // Set the VUS validation setting to VALIDACIO_VUS_I_DRUPAL.
    $this->config('diba_integration_vus.settings')
      ->set('diba_integration_vus_validation', VusManager::VALIDACIO_VUS_I_DRUPAL)
      ->save();

    // Create a user.
    $user = $this->drupalCreateUser();

    // Log out the admin user.
    $this->drupalLogout();

    // Attempt to log in with the created user.
    $this->drupalLogin($user);
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('user/' . $user->id());
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests user creation and login with VALIDACIO_VUS setting.
   */
  public function testCreateUserAndLoginWithValidacioVus() {
    // Set the VUS validation setting to VALIDACIO_VUS.
    $this->config('diba_integration_vus.settings')
      ->set('diba_integration_vus_validation', VusManager::VALIDACIO_VUS)
      ->save();

    // Create a user with the specified email.
    $user = $this->drupalCreateUser([], 'testvus', FALSE, ['mail' => 'test@test.com']);

    // Log out the admin user.
    $this->drupalLogout();

    // Attempt to log in with the created user, should fail.
    $this->drupalLogin($user);
  }

  /**
   * Tests user login with VALIDACIO_VUS_O_DRUPAL setting.
   */
  public function testLoginWithValidacioVusOrDrupal() {
    // Set the VUS validation setting to VALIDACIO_VUS_O_DRUPAL.
    $this->config('diba_integration_vus.settings')
      ->set('diba_integration_vus_validation', VusManager::VALIDACIO_VUS_O_DRUPAL)
      ->save();

    // Create a user with the email test@test.com.
    $user1 = $this->drupalCreateUser([], 'testvus1', FALSE, ['mail' => 'test@test.com']);

    // Create a user with the email test@diba.cat.
    $user2 = $this->drupalCreateUser([], 'testvus2', FALSE, ['mail' => 'test@diba.cat']);

    // Log out the admin user.
    $this->drupalLogout();

    // Attempt to log in with the user test@test.com.
    $this->drupalLogin($user1);
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('user/' . $user1->id());
    $this->assertSession()->statusCodeEquals(200);

    // Log out the user.
    $this->drupalLogout();

    // Attempt to log in with the user test@diba.cat, should fail.
    $this->drupalLogin($user2);
  }

}
